@@ -5,7 +5,7 @@
  | program/include/rcube_plugin_api.php                                  |
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2009, Roundcube Dev. - Switzerland                 |
+ | Copyright (C) 2008-2009, The Roundcube Dev Team                       |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -15,7 +15,7 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: rcube_plugin_api.php 3989 2010-09-25 13:03:53Z alec $
+ $Id: rcube_plugin_api.php 5151 2011-08-31 12:49:44Z alec $
 
 */
 
@@ -31,6 +31,7 @@ class rcube_plugin_api
   public $dir;
   public $url = 'plugins/';
   public $output;
+  public $config;
   
   public $handlers = array();
   private $plugins = array();
@@ -39,7 +40,6 @@ class rcube_plugin_api
   private $actionmap = array();
   private $objectsmap = array();
   private $template_contents = array();
-  
   private $required_plugins = array('filesystem_attachments');
   private $active_hook = false;
 
@@ -76,7 +76,7 @@ class rcube_plugin_api
   /**
    * This implements the 'singleton' design pattern
    *
-   * @return object rcube_plugin_api The one and only instance if this class
+   * @return rcube_plugin_api The one and only instance if this class
    */
   static function get_instance()
   {
@@ -86,8 +86,8 @@ class rcube_plugin_api
 
     return self::$instance;
   }
-  
-  
+
+
   /**
    * Private constructor
    */
@@ -95,8 +95,8 @@ class rcube_plugin_api
   {
     $this->dir = INSTALL_PATH . $this->url;
   }
-  
-  
+
+
   /**
    * Load and init all enabled plugins
    *
@@ -107,39 +107,13 @@ class rcube_plugin_api
   {
     $rcmail = rcmail::get_instance();
     $this->output = $rcmail->output;
+    $this->config = $rcmail->config;
 
-    $plugins_dir = dir($this->dir);
-    $plugins_dir = unslashify($plugins_dir->path);
     $plugins_enabled = (array)$rcmail->config->get('plugins', array());
-
     foreach ($plugins_enabled as $plugin_name) {
-      $fn = $plugins_dir . DIRECTORY_SEPARATOR . $plugin_name . DIRECTORY_SEPARATOR . $plugin_name . '.php';
-
-      if (file_exists($fn)) {
-        include($fn);
-
-        // instantiate class if exists
-        if (class_exists($plugin_name, false)) {
-          $plugin = new $plugin_name($this);
-          // check inheritance and task specification
-          if (is_subclass_of($plugin, 'rcube_plugin') && (!$plugin->task || preg_match('/^('.$plugin->task.')$/i', $rcmail->task))) {
-            $plugin->init();
-            $this->plugins[] = $plugin;
-          }
-        }
-        else {
-          raise_error(array('code' => 520, 'type' => 'php',
-	    'file' => __FILE__, 'line' => __LINE__,
-	    'message' => "No plugin class $plugin_name found in $fn"), true, false);
-        }
-      }
-      else {
-        raise_error(array('code' => 520, 'type' => 'php',
-	  'file' => __FILE__, 'line' => __LINE__,
-	  'message' => "Failed to load plugin file $fn"), true, false);
-      }
+      $this->load_plugin($plugin_name);
     }
-    
+
     // check existance of all required core plugins
     foreach ($this->required_plugins as $plugin_name) {
       $loaded = false;
@@ -149,48 +123,89 @@ class rcube_plugin_api
           break;
         }
       }
-      
-      // load required core plugin if no derivate was found
-      if (!$loaded) {
-        $fn = $plugins_dir . DIRECTORY_SEPARATOR . $plugin_name . DIRECTORY_SEPARATOR . $plugin_name . '.php';
 
-        if (file_exists($fn)) {
-          include_once($fn);
-          
-          if (class_exists($plugin_name, false)) {
-            $plugin = new $plugin_name($this);
-            // check inheritance
-            if (is_subclass_of($plugin, 'rcube_plugin')) {
-	      if (!$plugin->task || preg_match('/('.$plugin->task.')/i', $rcmail->task)) {
-                $plugin->init();
-                $this->plugins[] = $plugin;
-              }
-	      $loaded = true;
-            }
-          }
-        }
-      }
-      
+      // load required core plugin if no derivate was found
+      if (!$loaded)
+        $loaded = $this->load_plugin($plugin_name);
+
       // trigger fatal error if still not loaded
       if (!$loaded) {
         raise_error(array('code' => 520, 'type' => 'php',
-	  'file' => __FILE__, 'line' => __LINE__,
-	  'message' => "Requried plugin $plugin_name was not loaded"), true, true);
+          'file' => __FILE__, 'line' => __LINE__,
+          'message' => "Requried plugin $plugin_name was not loaded"), true, true);
       }
     }
 
     // register an internal hook
     $this->register_hook('template_container', array($this, 'template_container_hook'));
-    
+
     // maybe also register a shudown function which triggers shutdown functions of all plugin objects
   }
-  
-  
+
+
+  /**
+   * Load the specified plugin
+   *
+   * @param string Plugin name
+   * @return boolean True on success, false if not loaded or failure
+   */
+  public function load_plugin($plugin_name)
+  {
+    static $plugins_dir;
+
+    $rcmail = rcmail::get_instance();
+
+    if (!$plugins_dir) {
+      $dir = dir($this->dir);
+      $plugins_dir = unslashify($dir->path);
+    }
+
+    // plugin already loaded
+    if ($this->plugins[$plugin_name] || class_exists($plugin_name, false))
+      return true;
+
+    $fn = $plugins_dir . DIRECTORY_SEPARATOR . $plugin_name . DIRECTORY_SEPARATOR . $plugin_name . '.php';
+
+    if (file_exists($fn)) {
+      include($fn);
+
+      // instantiate class if exists
+      if (class_exists($plugin_name, false)) {
+        $plugin = new $plugin_name($this);
+        // check inheritance...
+        if (is_subclass_of($plugin, 'rcube_plugin')) {
+          // ... task, request type and framed mode
+          if ((!$plugin->task || preg_match('/^('.$plugin->task.')$/i', $rcmail->task))
+              && (!$plugin->noajax || (is_object($rcmail->output) && is_a($rcmail->output, 'rcube_template')))
+              && (!$plugin->noframe || empty($_REQUEST['_framed']))
+          ) {
+            $plugin->init();
+            $this->plugins[$plugin_name] = $plugin;
+          }
+          return true;
+        }
+      }
+      else {
+        raise_error(array('code' => 520, 'type' => 'php',
+          'file' => __FILE__, 'line' => __LINE__,
+          'message' => "No plugin class $plugin_name found in $fn"), true, false);
+      }
+    }
+    else {
+      raise_error(array('code' => 520, 'type' => 'php',
+        'file' => __FILE__, 'line' => __LINE__,
+        'message' => "Failed to load plugin file $fn"), true, false);
+    }
+
+    return false;
+  }
+
+
   /**
    * Allows a plugin object to register a callback for a certain hook
    *
-   * @param string Hook name
-   * @param mixed String with global function name or array($obj, 'methodname')
+   * @param string $hook Hook name
+   * @param mixed  $callback String with global function name or array($obj, 'methodname')
    */
   public function register_hook($hook, $callback)
   {
@@ -208,14 +223,14 @@ class rcube_plugin_api
         'file' => __FILE__, 'line' => __LINE__,
         'message' => "Invalid callback function for $hook"), true, false);
   }
-  
-  
+
+
   /**
    * Triggers a plugin hook.
    * This is called from the application and executes all registered handlers
    *
-   * @param string Hook name
-   * @param array Named arguments (key->value pairs)
+   * @param string $hook Hook name
+   * @param array $args Named arguments (key->value pairs)
    * @return array The (probably) altered hook arguments
    */
   public function exec_hook($hook, $args = array())
@@ -225,16 +240,16 @@ class rcube_plugin_api
 
     $args += array('abort' => false);
     $this->active_hook = $hook;
-    
+
     foreach ((array)$this->handlers[$hook] as $callback) {
       $ret = call_user_func($callback, $args);
       if ($ret && is_array($ret))
         $args = $ret + $args;
-      
+
       if ($args['abort'])
         break;
     }
-    
+
     $this->active_hook = false;
     return $args;
   }
@@ -243,10 +258,10 @@ class rcube_plugin_api
   /**
    * Let a plugin register a handler for a specific request
    *
-   * @param string Action name (_task=mail&_action=plugin.foo)
-   * @param string Plugin name that registers this action
-   * @param mixed Callback: string with global function name or array($obj, 'methodname')
-   * @param string Task name registered by this plugin
+   * @param string $action Action name (_task=mail&_action=plugin.foo)
+   * @param string $owner Plugin name that registers this action
+   * @param mixed  $callback Callback: string with global function name or array($obj, 'methodname')
+   * @param string $task Task name registered by this plugin
    */
   public function register_action($action, $owner, $callback, $task = null)
   {
@@ -255,7 +270,7 @@ class rcube_plugin_api
       $action = $task.'.'.$action;
     else if (strpos($action, 'plugin.') !== 0)
       $action = 'plugin.'.$action;
-    
+
     // can register action only if it's not taken or registered by myself
     if (!isset($this->actionmap[$action]) || $this->actionmap[$action] == $owner) {
       $this->actions[$action] = $callback;
@@ -273,7 +288,7 @@ class rcube_plugin_api
    * This method handles requests like _task=mail&_action=plugin.foo
    * It executes the callback function that was registered with the given action.
    *
-   * @param string Action name
+   * @param string $action Action name
    */
   public function exec_action($action)
   {
@@ -291,16 +306,16 @@ class rcube_plugin_api
   /**
    * Register a handler function for template objects
    *
-   * @param string Object name
-   * @param string Plugin name that registers this action
-   * @param mixed Callback: string with global function name or array($obj, 'methodname')
+   * @param string $name Object name
+   * @param string $owner Plugin name that registers this action
+   * @param mixed  $callback Callback: string with global function name or array($obj, 'methodname')
    */
   public function register_handler($name, $owner, $callback)
   {
     // check name
     if (strpos($name, 'plugin.') !== 0)
       $name = 'plugin.'.$name;
-    
+
     // can register handler only if it's not taken or registered by myself
     if (!isset($this->objectsmap[$name]) || $this->objectsmap[$name] == $owner) {
       $this->output->add_handler($name, $callback);
@@ -312,13 +327,13 @@ class rcube_plugin_api
         'message' => "Cannot register template handler $name; already taken by another plugin"), true, false);
     }
   }
-  
-  
+
+
   /**
    * Register this plugin to be responsible for a specific task
    *
-   * @param string Task name (only characters [a-z0-9_.-] are allowed)
-   * @param string Plugin name that registers this action
+   * @param string $task Task name (only characters [a-z0-9_.-] are allowed)
+   * @param string $owner Plugin name that registers this action
    */
   public function register_task($task, $owner)
   {
@@ -337,7 +352,7 @@ class rcube_plugin_api
       rcmail::$main_tasks[] = $task;
       return true;
     }
-    
+
     return false;
   }
 
@@ -345,6 +360,7 @@ class rcube_plugin_api
   /**
    * Checks whether the given task is registered by a plugin
    *
+   * @param string $task Task name
    * @return boolean True if registered, otherwise false
    */
   public function is_plugin_task($task)
@@ -357,16 +373,18 @@ class rcube_plugin_api
    * Check if a plugin hook is currently processing.
    * Mainly used to prevent loops and recursion.
    *
-   * @param string Hook to check (optional)
+   * @param string $hook Hook to check (optional)
    * @return boolean True if any/the given hook is currently processed, otherwise false
    */
   public function is_processing($hook = null)
   {
     return $this->active_hook && (!$hook || $this->active_hook == $hook);
   }
-  
+
   /**
    * Include a plugin script file in the current HTML page
+   *
+   * @param string $fn Path to script
    */
   public function include_script($fn)
   {
@@ -376,36 +394,51 @@ class rcube_plugin_api
     }
   }
 
+
   /**
    * Include a plugin stylesheet in the current HTML page
+   *
+   * @param string $fn Path to stylesheet
    */
   public function include_stylesheet($fn)
   {
     if ($this->output->type == 'html') {
       $src = $this->resource_url($fn);
-      $this->output->add_header(html::tag('link', array('rel' => "stylesheet", 'type' => "text/css", 'href' => $src)));
+      $this->output->include_css($src);
     }
   }
-  
+
+
   /**
    * Save the given HTML content to be added to a template container
+   *
+   * @param string $html HTML content
+   * @param string $container Template container identifier
    */
   public function add_content($html, $container)
   {
     $this->template_contents[$container] .= $html . "\n";
   }
-  
+
+
   /**
    * Callback for template_container hooks
+   *
+   * @param array $attrib
+   * @return array
    */
   private function template_container_hook($attrib)
   {
     $container = $attrib['name'];
     return array('content' => $attrib['content'] . $this->template_contents[$container]);
   }
-  
+
+
   /**
    * Make the given file name link into the plugins directory
+   *
+   * @param string $fn Filename
+   * @return string 
    */
   private function resource_url($fn)
   {
@@ -416,4 +449,3 @@ class rcube_plugin_api
   }
 
 }
-
